@@ -9,7 +9,7 @@
 #define GAPSITELIST_H_
 
 #include <stdlib.h>
-#include <R.h>
+#include "samtools/rdef.h"
 #include "bitmask.h"
 #include "samtools/sam.h"
 #include "samtools/bam.h"
@@ -85,14 +85,14 @@ static R_INLINE site_list_element* copy_site_list_element(const site_list_elemen
 	return el;
 }
 
-void static R_INLINE site_list_add_cs(struct site_list_element* el,sle_type lcl,index_type rcs,sle_type mcl)
+static R_INLINE void site_list_add_cs(struct site_list_element* el,sle_type lcl,index_type rcs,sle_type mcl)
 {
 	++(el->nAligns);
 	// Maximum of right cigar size
 	if(el->r_cigar_size<rcs)
 		el->r_cigar_size=rcs;
-	r_addVal(&(el->lcl),lcl);
-	r_addVal_f(&(el->mcl),mcl);
+	r_addVal(&(el->lcl), (value_type)lcl);
+	r_addVal_f(&(el->mcl),(value_type) mcl);
 }
 
 // static _inline_ void site_list_el_destroy(site_list_element* el) { free(el); }
@@ -549,7 +549,7 @@ void list_gap_sites(site_list *l,const bam1_t* align)
 
 	cigar    = bam1_cigar(align);
 	// Add size of first cigar to position to get 1-based left stop (BAM is 0-based)
-	position = align->core.pos + (cigar[0] >> BAM_CIGAR_SHIFT);
+	position =  ((uint32_t)align->core.pos) + BC_RIGHT_SHIFT(cigar[0]); //(cigar[0] >> BAM_CIGAR_SHIFT);
 	n_cigar  = align->core.n_cigar;
 	op = cigar[0] & BAM_CIGAR_MASK;
 	if((op!= BAM_CMATCH))
@@ -573,7 +573,7 @@ void list_gap_sites(site_list *l,const bam1_t* align)
 		// Add size of first cigar to position to get *1-based* left stop
 		// BAM is *0-based*
 		// position always points to *1-based* last nucleotide of active cigar item
-		position = align->core.pos + (cigar[0] >> BAM_CIGAR_SHIFT);
+		position =  ((uint32_t)align->core.pos) + BC_RIGHT_SHIFT(cigar[0]);//(cigar[0] >> BAM_CIGAR_SHIFT);
 
 		for(i=1;i<(n_cigar-1);++i)
 		{
@@ -586,7 +586,7 @@ void list_gap_sites(site_list *l,const bam1_t* align)
 				++(l->nAlignGaps);
 				// N -> Add gap to list
 				lend=position;
-				gap_len=cigar[i] >> BAM_CIGAR_SHIFT;
+				gap_len= BC_RIGHT_SHIFT(cigar[i]); //cigar[i] >> BAM_CIGAR_SHIFT;
 
 				position += gap_len;
 				// position now points to *1-based* last nuc of gap
@@ -598,7 +598,7 @@ void list_gap_sites(site_list *l,const bam1_t* align)
 				if((cigar[i-1] & BAM_CIGAR_MASK)!=BAM_CMATCH)
 					return;
 				// Add left cigar data
-				left_len=cigar[i-1]>>BAM_CIGAR_SHIFT;
+				left_len=BC_RIGHT_SHIFT(cigar[i-1]); //cigar[i-1]>>BAM_CIGAR_SHIFT;
 				// Adding one -> always positive values -> output as factor
 				//g.left_cigar_type=BAM_CMATCH+1;
 
@@ -606,7 +606,7 @@ void list_gap_sites(site_list *l,const bam1_t* align)
 				if((cigar[i+1] & BAM_CIGAR_MASK)!=BAM_CMATCH)
 					return;
 				// Add right cigar data
-				right_len=cigar[i+1]>>BAM_CIGAR_SHIFT;
+				right_len=BC_RIGHT_SHIFT(cigar[i+1]);  //cigar[i+1]>>BAM_CIGAR_SHIFT;
 				// Adding one -> always positive values -> output as factor
 				// g.right_cigar_type=BAM_CMATCH+1;
 
@@ -616,25 +616,25 @@ void list_gap_sites(site_list *l,const bam1_t* align)
 						lend,							// lend
 						rstart,							// rstart
 						gap_len,						// gap_len
-						left_len,						// lcl
+						(unsigned char) left_len,		// lcl
 						right_len,						// rcs
-						min_mtc                         // mcl
+						(unsigned char) min_mtc         // mcl
 						);
 
 				// next cigar can also be processed because we know it's an M
 				++i;
-				position+=(cigar[i] >> BAM_CIGAR_SHIFT);
+				position+= BC_RIGHT_SHIFT(cigar[i]); //(cigar[i] >> BAM_CIGAR_SHIFT);
 				// position now points to *1-based* last nuc of match
 			}
 			else if(op == BAM_CMATCH)
 			{
 				// position then points on rightmost nuc of exon
-				position +=(cigar[i] >> BAM_CIGAR_SHIFT);
+				position +=BC_RIGHT_SHIFT(cigar[i]); //(cigar[i] >> BAM_CIGAR_SHIFT);
 			}
 			else if(op == BAM_CDEL)
 			{
 				// shift position
-				position +=(cigar[i] >> BAM_CIGAR_SHIFT);
+				position +=BC_RIGHT_SHIFT(cigar[i]); //(cigar[i] >> BAM_CIGAR_SHIFT);
 			}
 		}
 		//printf("\n");
