@@ -7,7 +7,11 @@
 #include "kstring.h"
 #include "sam_header.h"
 
+#ifdef R_CRAN
 #include <R.h>
+#else
+#define R_INLINE inline
+#endif
 
 int bam_is_be = 0, bam_verbose = 2;
 char *bam_flag2char_table = "pPuUrR12sfd\0\0\0\0\0";
@@ -21,9 +25,9 @@ uint32_t bam_calend(const bam1_core_t *c, const uint32_t *cigar)
 	uint32_t k, end;
 	end = c->pos;
 	for (k = 0; k < c->n_cigar; ++k) {
-		int op = cigar[k] & BAM_CIGAR_MASK;
+		int op = (int) cigar[k] & BAM_CIGAR_MASK;
 		if (op == BAM_CMATCH || op == BAM_CDEL || op == BAM_CREF_SKIP)
-			end += cigar[k] >> BAM_CIGAR_SHIFT;
+			end += BC_RIGHT_SHIFT(cigar[k]); // cigar[k] >> BAM_CIGAR_SHIFT;
 	}
 	return end;
 }
@@ -33,9 +37,9 @@ int32_t bam_cigar2qlen(const bam1_core_t *c, const uint32_t *cigar)
 	uint32_t k;
 	int32_t l = 0;
 	for (k = 0; k < c->n_cigar; ++k) {
-		int op = cigar[k] & BAM_CIGAR_MASK;
+		int op = (int) cigar[k] & BAM_CIGAR_MASK;
 		if (op == BAM_CMATCH || op == BAM_CINS || op == BAM_CSOFT_CLIP || op == BAM_CEQUAL || op == BAM_CDIFF)
-			l += cigar[k] >> BAM_CIGAR_SHIFT;
+			l += BC_RIGHT_SHIFT(cigar[k]); // cigar[k] >> BAM_CIGAR_SHIFT;
 	}
 	return l;
 }
@@ -77,19 +81,15 @@ bam_header_t *bam_header_read(bamFile fp)
 	// check EOF
 	i = bgzf_check_EOF(fp);
 	if (i < 0) {
-		// If the file is a pipe, checking the EOF marker will *always* fail
-		// with ESPIPE.  Suppress the error message in this case.
-		// REP: if (errno != ESPIPE) perror("[bam_header_read] bgzf_check_EOF");
-		if (errno != ESPIPE) Rprintf("[bam_reader_read] bgzf_check_EOF %s",strerror( errno ));
-
+		if (errno != ESPIPE)
+			Rprintf("[bam_reader_read] bgzf_check_EOF %s",strerror( errno ));
 	}
 	else if (i == 0)
-		// REP: fprintf(stderr, "[bam_header_read] EOF marker is absent. The input is probably truncated.\n");
 		Rprintf("[bam_header_read] EOF marker is absent. The input is probably truncated.\n");
+
 	// read "BAM1"
 	magic_len = bam_read(fp, buf, 4);
 	if (magic_len != 4 || strncmp(buf, "BAM\001", 4) != 0) {
-		// REP: fprintf(stderr, "[bam_header_read] invalid BAM binary header (this is not a BAM file).\n");
 		Rprintf("[bam_header_read] invalid BAM binary header (this is not a BAM file).\n");
 		return 0;
 	}
@@ -215,6 +215,16 @@ int bam_read1(bamFile fp, bam1_t *b)
 	if (bam_read(fp, b->data, b->data_len) != b->data_len) return -4;
 	b->l_aux = b->data_len - c->n_cigar * 4 - c->l_qname - c->l_qseq - (c->l_qseq+1)/2;
 	if (bam_is_be) swap_endian_data(c, b->data_len, b->data);
+
+
+#ifdef BAM1_ADD_CIGAR
+	// + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + //
+	// Fill additional cigar field with copy of cigar data
+	// + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + //
+	COPY_CIGAR_VALUES(b);
+	// + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + //
+#endif
+
 	return 4 + block_len;
 }
 
@@ -344,7 +354,6 @@ char *bam_format1(const bam_header_t *header, const bam1_t *b)
 void bam_view1(const bam_header_t *header, const bam1_t *b)
 {
 	char *s = bam_format1(header, b);
-	//puts(s);
 	Rprintf("%s\n",s);
 	free(s);
 }
