@@ -2,7 +2,11 @@
 #define BAM_ENDIAN_H
 
 #include <stdint.h>
-#include <R.h>
+
+/* Settings for optional embedding into R*/
+#include "rdef.h"
+
+
 
 static R_INLINE int bam_is_big_endian()
 {
