@@ -4,7 +4,7 @@
 #include "khash.h"
 #include "ksort.h"
 #include "bam_endian.h"
-#include <R.h>
+
 //#ifdef _USE_KNETFILE
 //#include "knetfile.h"
 //#endif
@@ -161,7 +161,8 @@ bam_index_t *bam_index_core(bamFile fp)
 	uint64_t save_off, last_off, n_mapped, n_unmapped, off_beg, off_end, n_no_coor;
 
 	idx = (bam_index_t*)calloc(1, sizeof(bam_index_t));
-	b = (bam1_t*)calloc(1, sizeof(bam1_t));
+	b= bam_init1();
+	//b = (bam1_t*)calloc(1, sizeof(bam1_t));
 	h = bam_header_read(fp);
 	c = &b->core;
 
@@ -234,7 +235,9 @@ bam_index_t *bam_index_core(bamFile fp)
 	}
 	// REP: if (ret < -1) fprintf(stderr, "[bam_index_core] truncated file? Continue anyway. (%d)\n", ret);
 	if (ret < -1) Rprintf("[bam_index_core] truncated file? Continue anyway. (%d)\n", ret);
-	free(b->data); free(b);
+
+	bam_destroy1(b);
+	//free(b->data); free(b);
 	idx->n_no_coor = n_no_coor;
 	return idx;
 }
@@ -390,14 +393,26 @@ bam_index_t *bam_index_load_local(const char *_fn)
 {
 	FILE *fp;
 	char *fnidx, *fn;
+	size_t len;
 
 	if (strstr(_fn, "ftp://") == _fn || strstr(_fn, "http://") == _fn) {
 		const char *p;
 		int l = strlen(_fn);
 		for (p = _fn + l - 1; p >= _fn; --p)
 			if (*p == '/') break;
-		fn = strdup(p + 1);
-	} else fn = strdup(_fn);
+
+		//fn = str_dup(p + 1);
+		len = strlen(p + 1) + 1;
+		fn = malloc(len);
+		memcpy(fn,p + 1,len);
+	}
+	else
+	{
+		//fn = str_dup(_fn);
+		len = strlen(_fn) + 1;
+		fn  = malloc(len);
+		memcpy(fn,_fn,len);
+	}
 	fnidx = (char*)calloc(strlen(fn) + 5, 1);
 	strcpy(fnidx, fn); strcat(fnidx, ".bai");
 	fp = fopen(fnidx, "rb");
@@ -480,25 +495,31 @@ int bam_index_build2(const char *fn, const char *_fnidx)
 	FILE *fpidx;
 	bamFile fp;
 	bam_index_t *idx;
+	size_t len;
+
 	if ((fp = bam_open(fn, "r")) == 0) {
-		// REP: fprintf(stderr, "[bam_index_build2] fail to open the BAM file.\n");
 		Rprintf("[bam_index_build2] fail to open BAM file: '%s'\n",fn);
 		return -1;
 	}
 	idx = bam_index_core(fp);
 	bam_close(fp);
 	if(idx == 0) {
-		// REP: fprintf(stderr, "[bam_index_build2] fail to index the BAM file.\n");
 		Rprintf("[bam_index_build2] fail to index the BAM file.\n");
 		return -1;
 	}
 	if (_fnidx == 0) {
 		fnidx = (char*)calloc(strlen(fn) + 5, 1);
 		strcpy(fnidx, fn); strcat(fnidx, ".bai");
-	} else fnidx = strdup(_fnidx);
+	}
+	else
+	{
+		//fnidx = str_dup(_fnidx);
+		len = strlen(_fnidx) + 1;
+		fnidx = malloc(len);
+		memcpy(fnidx,_fnidx,len);
+	}
 	fpidx = fopen(fnidx, "wb");
 	if (fpidx == 0) {
-		// REP: fprintf(stderr, "[bam_index_build2] fail to create the index file.\n");
 		Rprintf("[bam_index_build2] fail to create the index file.\n");
 		free(fnidx);
 		return -1;
@@ -518,7 +539,6 @@ int bam_index_build(const char *fn)
 int bam_index(int argc, char *argv[])
 {
 	if (argc < 2) {
-		// REP: fprintf(stderr, "Usage: samtools index <in.bam> [out.index]\n");
 		Rprintf("Usage: samtools index <in.bam> [out.index]\n");
 		return 1;
 	}
