@@ -27,7 +27,11 @@
 #include <stdint.h>
 #include <stdio.h>
 #include <zlib.h>
-#include <R.h>
+
+/* Settings for optional embedding into R*/
+#include "rdef.h"
+
+
 //#ifdef _USE_KNETFILE
 //#include "knetfile.h"
 //#endif
@@ -106,7 +110,7 @@ int bgzf_write(BGZF* fp, const void* data, int length);
  * Return value is non-negative on success.
  * Returns -1 on error.
  */
-#define bgzf_tell(fp) ((fp->block_address << 16) | (fp->block_offset & 0xFFFF))
+#define bgzf_tell(fp) ((fp->block_address << 16) | ((int64_t)fp->block_offset & 0xFFFF))
 
 /*
  * Set the file to read from the location specified by pos, which must
@@ -139,10 +143,10 @@ static R_INLINE int bgzf_getc(BGZF *fp)
 {
 	int c;
 	if (fp->block_offset >= fp->block_length) {
-		if (bgzf_read_block(fp) != 0) return -2; /* error */
-		if (fp->block_length == 0) return -1; /* end-of-file */
+		if (bgzf_read_block(fp) != 0) return -2; 	/* error */
+		if (fp->block_length == 0) return -1; 		/* end-of-file */
 	}
-	c = ((unsigned char*)fp->uncompressed_block)[fp->block_offset++];
+	c = ((Bytef*)fp->uncompressed_block)[fp->block_offset++];
     if (fp->block_offset == fp->block_length) {
 #ifdef _USE_KNETFILE
         fp->block_address = knet_tell(fp->x.fpr);
